"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const timers_1 = require("timers");
const child_process_1 = require("child_process");
const queue_1 = require("./queue");
const DESTROY_INTERVAL = 500;
const availableWorkers = new queue_1.Queue();
const awaitingJobs = new queue_1.Queue();
const workerCallback = (worker) => {
    return (res) => {
        availableWorkers.add(worker);
        if (process.send) {
            process.send(res);
        }
    };
};
process.on('message', (msg) => {
    if (msg.type === 'create' && msg.args.length > 0) {
        for (let i = 0; i < msg.args[0]; i++) {
            const worker = child_process_1.fork(path_1.default.resolve(__dirname, './worker.js'));
            const newWorker = {
                id: i + 1,
                worker,
            };
            worker.on('message', workerCallback(newWorker));
            availableWorkers.add(newWorker);
        }
    }
    if (msg.type === 'job') {
        awaitingJobs.add(msg);
    }
    if (msg.type === 'destroy') {
        const destroyTimer = setInterval(() => {
            const job = awaitingJobs.pop();
            if (!job) {
                let worker = availableWorkers.pop();
                while (worker) {
                    worker.worker.kill('SIGINT');
                    worker = availableWorkers.pop();
                }
                timers_1.clearInterval(destroyTimer);
            }
        }, DESTROY_INTERVAL);
    }
});
// main event loop
setInterval(() => {
    const worker = availableWorkers.pop();
    if (worker) {
        const job = awaitingJobs.pop();
        if (job) {
            worker.worker.send(job);
        }
        else {
            availableWorkers.add(worker);
        }
    }
}, 1);
