"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoolSingleton = exports.Pool = void 0;
const tslib_1 = require("tslib");
const os_1 = tslib_1.__importDefault(require("os"));
const path_1 = tslib_1.__importDefault(require("path"));
const child_process_1 = require("child_process");
const uuid_1 = require("uuid");
const MAX_PROCESSES = os_1.default.cpus().length;
const TO_SECONDS = 1000;
const MINUTE_SECONDS = 60;
const listeners = new Map();
class Pool {
    constructor(numProcesses) {
        this.id = uuid_1.v4();
        this.master = child_process_1.fork(path_1.default.resolve(__dirname, './master-worker.js'));
        this.instantiate();
        this.send({
            id: this.id,
            type: 'create',
            fn: (() => { }).toString(),
            args: [numProcesses || Pool.MAX_PROCESSES],
        });
    }
    instantiate() {
        this.master.on('message', (result) => {
            const event = listeners.get(result.id);
            if (event) {
                clearTimeout(event.timeout);
                event.resolver(result.data);
            }
        });
    }
    send(job) {
        this.master.send(job);
    }
    addJob(fn, ...args) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const id = uuid_1.v4();
            this.send({ fn: fn.toString(), args, id, type: 'job' });
            return new Promise((resolver, reject) => {
                const timeout = setTimeout(() => {
                    reject(new Error(`timeout reached. timeout ${Pool.TIMEOUT}`));
                }, Pool.TIMEOUT * TO_SECONDS);
                listeners.set(id, {
                    timeout,
                    resolver,
                });
            });
        });
    }
    kill() {
        this.send({
            id: this.id,
            type: 'destroy',
            fn: (() => { }).toString(),
            args: [],
        });
        this.master.kill('SIGINT');
    }
}
exports.Pool = Pool;
Pool.MAX_PROCESSES = MAX_PROCESSES;
Pool.TIMEOUT = MINUTE_SECONDS; // default 60 seconds
class PoolSingleton extends Pool {
    constructor(numProcesses) {
        super(numProcesses);
    }
    static setTimeout(seconds) {
        PoolSingleton.TIMEOUT = seconds;
    }
    static setMaxProcesses(maxProcesses) {
        PoolSingleton.MAX_PROCESSES = maxProcesses;
    }
    static getInstance() {
        if (!this.instance) {
            this.instance = new PoolSingleton(PoolSingleton.MAX_PROCESSES);
        }
        return this.instance;
    }
}
exports.PoolSingleton = PoolSingleton;
PoolSingleton.TIMEOUT = MINUTE_SECONDS; // default 60 seconds
PoolSingleton.MAX_PROCESSES = MAX_PROCESSES;
PoolSingleton.instance = null;
exports.default = PoolSingleton;
